
#include <QApplication>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <HelloDlg.h>
#include <Ice/Ice.h>
#include <QEvent>
#include <iomanip>

using namespace std;

class HelloEvent : public QEvent
{
public:

    HelloEvent(bool success, QString exception = "") : QEvent(QEvent::Type(QEvent::User + 1)), _success(success), _exception(exception)
    {
    }

    bool success()
    {
        return _success;
    }

    QString exception()
    {
        return _exception;
    }

private:

    bool _success;
    QString _exception;
};

class AMI_Hello_sayHelloI : public Demo::AMI_Hello_sayHello
{
public:
    AMI_Hello_sayHelloI(QObject* listener) : _listener(listener)
    {
    }
    virtual void ice_response()
    {
        if(_listener)
        {
            QApplication::postEvent(_listener, new HelloEvent(true));
        }
    }

    virtual void ice_exception(const Ice::Exception& ex)
    {
        if(_listener)
        {
            ostringstream out;
            out << ex;
            QApplication::postEvent(_listener, new HelloEvent(false, out.str().c_str()));
        }
    }

private:

    QObject* _listener;
};


HelloDlg::HelloDlg(const Demo::HelloPrx& helloPrx)
    : _helloPrx(helloPrx)
{
    _txtReply = new QTextEdit();
    _txtReply->setReadOnly(true);
    _pbtSayHello = new QPushButton();
    _pbtSayHello->setText(tr("Say hello"));


    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->addWidget(_txtReply);

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->addWidget(_pbtSayHello);
    
    vLayout->addLayout(hLayout);
    setLayout(vLayout);
    
    connect(_pbtSayHello, SIGNAL(clicked()), this, SLOT(sayHello()));

    installEventFilter(new HelloEventFilter(this));
}

void
HelloDlg::sayHello()
{
    try
    {
        _helloPrx->sayHello_async(new AMI_Hello_sayHelloI(this), 0);
    }
    catch(const Ice::Exception& ex)
    {
        ostringstream out;
        out << ex;
        writeText(out.str().c_str());
    }
}

void
HelloDlg::writeText(QString text)
{
    _txtReply->setText(text);
}

HelloEventFilter::HelloEventFilter(QObject* parent)
    :QObject(parent)
{
}

bool
HelloEventFilter::eventFilter(QObject *o,QEvent *e )
{
    if(e->type() == QEvent::Type(QEvent::User + 1)) 
    {
        //Process hello event
        HelloEvent* helloEvent =(HelloEvent*)e;
        HelloDlg* helloDlg = (HelloDlg*)o;
        if(helloEvent->success())
        {
            helloDlg->writeText("Hello call success");
        }
        else
        {
            helloDlg->writeText("Hello call failed with exception:\n" + helloEvent->exception());
        }
        return true; // eat event
    }
    else 
    {
        // standard event processing
        return false;
    }
}


int main(int argc, char** argv)
{
    int status;
    QApplication app(argc, argv);
    Ice::CommunicatorPtr ic;
    try
    {
        ic = Ice::initialize(argc, argv);
        Demo::HelloPrx helloPrx = Demo::HelloPrx::uncheckedCast(ic->propertyToProxy("Hello.Proxy"));
        HelloDlg* helloDlg = new HelloDlg(helloPrx);
        helloDlg->show();
        status = app.exec();
    }
    catch(const Ice::Exception& ex)
    {
        std::cerr << ex << endl;
        status = EXIT_FAILURE;
    }
    catch(const std::exception& ex)
    {
        std::cerr << ex.what() << endl;
        status = EXIT_FAILURE;
    }
    if(ic)
    {
        ic->destroy();
    }
    return status;
}
